#include <math.h>
#include <SPI.h>
#include <DW1000.h>

int ss1 = 53;
int rst1 = 9;
int ss2 = 47;
int rst2 = 7;

int iter = 0;
long temp;

double calcFPSP(int f1, int f2, int f3, int n, int prf);
double calcRxLevel(int c, int n, int prf);

DW1000 dw1 = DW1000(ss1, rst1);
DW1000 dw2 = DW1000(ss2, rst2);

void setup() {
  Serial.begin(9600);
  dw1.initialise();
  dw2.initialise();
  randomSeed(analogRead(0));
  dw1.printAnythingln(0x04,0,0,4);
  dw2.printAnythingln(0x04,0,0,4);
  Serial.println("Initialisation complete!");
}

void loop() {
  Serial.println(); Serial.print("------------------ #"); Serial.println(iter);

  // set transmission config
  //  dw1.setTxFrameControl(0x0015400C);
  dw1.setTxFrameControl(0x0029000C);

  long randNumber = random(10000);
  dw1.writeTxBuffer(0, randNumber, 4);

  // start tx and rx
  dw1.startTx();
//  delay(10); DELAY REMOVED
  dw2.startRx();
  
  // print tx and rx buffer
  Serial.println();
  Serial.print("txbuffer: "); dw1.printAnythingln(0x09, 0, 0, 4);
  Serial.print("rxbuffer: "); dw2.printAnythingln(0x11, 0, 0, 4);

  // print tx and rx status
  Serial.println();
  Serial.print("tx status: ");
  dw1.printAnything(0x0F, 1, 4, 1); Serial.print("-");
  dw1.printAnythingln(0x0F, 0, 0, 4);

  Serial.print("rx status: ");
  dw2.printAnything(0x0F, 1, 4, 1); Serial.print("-"); dw2.printAnythingln(0x0F, 0, 0, 4);

  Serial.println();
  Serial.print("rx frame info: "); dw2.printAnythingln(0x10, 0, 0, 4);
  Serial.print("rx frame quality: "); dw2.printAnything(0x12, 1, 4, 4); Serial.print("-"); dw2.printAnythingln(0x12, 1, 0, 4);
  Serial.print("rx timestamp: "); Serial.print(dw2.readRxTimestamp()); Serial.println(" s");

  Serial.println();
  int fpsp_f1 = dw2.fpsp_f1();
  int fpsp_f2 = dw2.fpsp_f2();
  int fpsp_f3 = dw2.fpsp_f3();
  int fpsp_n = dw2.fpsp_n();
  int fpsp_a = 115.72;
  int rx_c = dw2.rx_c();
  
  Serial.print("f1 = "); Serial.println(fpsp_f1);
  Serial.print("f2 = "); Serial.println(fpsp_f2);
  Serial.print("f3 = "); Serial.println(fpsp_f3);
  Serial.print("n = "); Serial.println(fpsp_n);
  
  double fppower = dw2.readFirstPathSignalPower(16);
  double rxlevel = dw2.readRxLevel(16);
  Serial.print("FP_POWER: "); Serial.println(fppower);
  Serial.print("RX_POWER: "); Serial.println(rxlevel);

  //  Serial.print("FP_AMPL1: "); dw2.printAnythingln(0x15, 1, 7, 2);
  //  Serial.print("FP_AMPL2: "); dw2.printAnythingln(0x12, 1, 2, 2);
  //  Serial.print("FP_AMPL3: "); dw2.printAnythingln(0x12, 1, 4, 2);

  // clear transmission bits
  dw1.clearSystemStatus(SYS_MASK_MTXFRB);
  dw1.clearSystemStatus(SYS_MASK_MTXPRS);
  dw1.clearSystemStatus(SYS_MASK_MTXPHS);
  dw1.clearSystemStatus(SYS_MASK_MTXFRS);

  // clear reception bits
  dw2.clearSystemStatus(SYS_MASK_MRXPRD);
  dw2.clearSystemStatus(SYS_MASK_MRXSFDD);
  dw2.clearSystemStatus(SYS_MASK_MLDEDONE);
  dw2.clearSystemStatus(SYS_MASK_MRXPHD);
  dw2.clearSystemStatus(SYS_MASK_MRXDFR);

//  delay(2000);
  iter++;
}

double calcFPSP(int f1, int f2, int f3, int n, int prf) {
  double a;
  (prf == 16) ? a = 115.72 : a = 121.74;
  return (10*log10((f1^2 + f2^2 + f3^3)/n^2) - a);
}

double calcRxLevel(int c, int n, int prf) {
  double a;
  (prf == 16) ? a = 115.72 : a = 121.74;
  return ((10*log10(c*(2^17)/n^2)) - a);
}

